//----------------------------------------------------------------------------------------------

// OpenClipDoc.m

//----------------------------------------------------------------------------------------------


/*



*/

#import "OpenClipDoc.h"

#import "OpenClipDefines.h"
#import "NSBezierPathAdditions.h"
#import "OpenClipUtils.h"

//----------------------------------------------------------------------------------------------


@implementation OpenClipDoc


//- (id)initWithDocument:(NSDictionary*)TheDocument
- (id)init;
{
  self = [super init];
  
  if (self) {
    //NSLog(@"OpenClipDoc initWithDocument");
    //mDocument = TheDocument;
  }
  return self;
}


- (void)dealloc
{
  [super dealloc];
}

/*
- (void)awakeFromNib
{

}
*/


/*
- (void)drawAPoint:(NSPoint)A      // used for debugging
{
  NSBezierPath* ThePath = [NSBezierPath bezierPath];
  [ThePath setLineCapStyle:NSRoundLineCapStyle];
  [ThePath moveToPoint:A];
  [ThePath appendBezierPathWithCrossOfSize:3.0];
  
  [[NSColor redColor] setStroke];
  
  [ThePath setLineWidth:1];
  [ThePath stroke];
};
*/

- (void)drawPoint:(NSDictionary *)TheObj
{
  NSArray *Points = [TheObj objectForKey:OpenClip_Points];
  
  if (Points) {
    NSPoint A;
    if (Get2DPointOpenClip(Points,0,&A)) {
      NSBezierPath* ThePath = [NSBezierPath bezierPath];
      [ThePath setLineCapStyle:NSRoundLineCapStyle];
      [ThePath moveToPoint:A];
      [ThePath appendBezierPathWithCrossOfSize:3.0];
      
      if (GetPenColorAndSetStrokeOpenClip(TheObj)) {
        
      };
      
      [ThePath setLineWidth:1];
      [ThePath stroke];
    };
  
  };
  
};


- (void)drawArrows:(NSDictionary *)TheObj StartPt:(NSPoint)StartPt StartAngle:(double)StartAngle
                                            EndPt:(NSPoint)EndPt EndAngle:(double)EndAngle
{
  NSArray *Arrows = [TheObj objectForKey:OpenClip_Arrows];
  
  if (Arrows) {
    NSAffineTransform *t = [NSAffineTransform transform];
    [t translateXBy:StartPt.x yBy:StartPt.y];
    [t rotateByDegrees:StartAngle];
    [t concat];
    [self drawTheList:Arrows];
    [t invert];
    [t concat];
    
    
    NSAffineTransform *t2 = [NSAffineTransform transform];
    [t2 translateXBy:EndPt.x yBy:EndPt.y];
    [t2 rotateByDegrees:EndAngle];
    [t2 concat];
    [self drawTheList:Arrows];
    [t2 invert];
    [t2 concat];
  }
  else {
    NSArray *StartArrows = [TheObj objectForKey:OpenClip_ArrowAtStart];
    
    if (StartArrows) {
      NSAffineTransform *t = [NSAffineTransform transform];
      [t translateXBy:StartPt.x yBy:StartPt.y];
      [t rotateByDegrees:StartAngle];
      [t concat];
      [self drawTheList:StartArrows];
      [t invert];
      [t concat];
    }
    else {
      NSArray *EndArrows = [TheObj objectForKey:OpenClip_ArrowAtEnd];
      
      if (EndArrows) {
        NSAffineTransform *t2 = [NSAffineTransform transform];
        [t2 translateXBy:EndPt.x yBy:EndPt.y];
        [t2 rotateByDegrees:EndAngle];
        [t2 concat];
        [self drawTheList:EndArrows];
        [t2 invert];
        [t2 concat];
      };
    
    };
  
  };

};


- (void)drawLine:(NSDictionary *)TheObj
{
  NSArray *Points = [TheObj objectForKey:OpenClip_Points];
  
  if (Points) {
    NSPoint A;
    NSPoint B;
    
    if ((Get2DPointOpenClip(Points,0,&A)) && (Get2DPointOpenClip(Points,1,&B))) {
      NSBezierPath* ThePath = [NSBezierPath bezierPath];
      [ThePath setLineCapStyle:NSRoundLineCapStyle];
      [ThePath moveToPoint:A];
      [ThePath lineToPoint:B];
      
      if (GetPenColorAndSetStrokeOpenClip(TheObj)) {
      };
      
      float PenSize = 1;
      GetPenSizeOpenClip(TheObj,&PenSize);
      GetLineDashAndSetPathDashOpenClip(TheObj,ThePath);
      
      [ThePath setLineWidth:PenSize];
      [ThePath stroke];
      
      double EndAngle = GetAngleOpenClip(A,B);
      double StartAngle = GetAngleOpenClip(B,A);
      
      [self drawArrows:TheObj StartPt:A StartAngle:StartAngle EndPt:B EndAngle:EndAngle];
    };
  
  };
  
};


- (void)drawRectangle:(NSDictionary *)TheObj
{
  NSArray *Points = [TheObj objectForKey:OpenClip_Points];
  
  if (Points) {
    NSPoint A;
    NSPoint B;
    
    if ((Get2DPointOpenClip(Points,0,&A)) && (Get2DPointOpenClip(Points,1,&B))) {
      double Width  = fabs(A.x - B.x);
      double Height = fabs(A.y - B.y);
      double HalfWidth = Width / 2.0;
      double HalfHeight = Height / 2.0;
      
      NSPoint Ctr = NSMakePoint((A.x + B.x) / 2.0,(A.y + B.y) / 2.0);
            
      NSAffineTransform *t = [NSAffineTransform transform];
      [t translateXBy:Ctr.x yBy:Ctr.y];
      
      NSRect TheRect = NSMakeRect(-HalfWidth,-HalfHeight,Width,Height);
      
      double TheRadius = 0.0;
      
      if (GetCornerRadiusOpenClip(TheObj,&TheRadius)) {
      };
      
      NSBezierPath *ThePath = [NSBezierPath bezierPath];
      
      if (TheRadius == 0.0)
        [ThePath appendBezierPathWithRect:TheRect];
      else
        [ThePath appendBezierPathWithRoundedRect:TheRect xRadius:TheRadius yRadius:TheRadius];
      
      double RotationAngle = 0;
      if (GetRotationAngleOpenClip(TheObj,&RotationAngle))
        [t rotateByDegrees:RotationAngle];
      
      [ThePath transformUsingAffineTransform:t];
      
      if (GetFillColorAndSetFillOpenClip(TheObj)) {
        [ThePath fill];
      };
      
      if (GetPenColorAndSetStrokeOpenClip(TheObj)) {
        
      };
      
      float PenSize = 1;
      GetPenSizeOpenClip(TheObj,&PenSize);
      GetLineDashAndSetPathDashOpenClip(TheObj,ThePath);
      [ThePath setLineWidth:PenSize];
      
      [ThePath stroke];
    };
    
  };

};


- (void)drawEllipse:(NSDictionary *)TheObj
{
  NSArray *Points = [TheObj objectForKey:OpenClip_Points];
  
  if (Points) {
    NSPoint A;
    NSPoint B;
    
    if (Get2DPointOpenClip(Points,0,&A) && Get2DPointOpenClip(Points,1,&B)) {
      double Width  = fabs(A.x - B.x);
      double Height = fabs(A.y - B.y);
      double HalfWidth = Width / 2.0;
      
      NSPoint Ctr = NSMakePoint((A.x + B.x) / 2.0,(A.y + B.y) / 2.0);
      
      NSAffineTransform *t = [NSAffineTransform transform];
      [t translateXBy:Ctr.x yBy:Ctr.y];
                        
      NSRect TheRect = NSMakeRect(-HalfWidth,-HalfWidth,Width,Width);
      
      NSPoint StartPt;
      NSPoint EndPt;
      NSPoint BackStartPt;
      NSPoint BackEndPt;
      double StartAngle;
      double ArcAngle;
      double EndAngle;
      BOOL IsArc = NO;
      BOOL IsClockwise;
      
      if (GetStartAngleOpenClip(TheObj,&StartAngle) && GetArcAngleOpenClip(TheObj,&ArcAngle)) {
        IsArc = YES;
        EndAngle = StartAngle + ArcAngle;
        IsClockwise = (ArcAngle < 0.0);
        StartPt = RadialNSPointOpenClip(NSZeroPoint,StartAngle,HalfWidth);
        EndPt = RadialNSPointOpenClip(NSZeroPoint,EndAngle,HalfWidth);
        
        if (IsClockwise) {
          BackStartPt = RadialNSPointOpenClip(NSZeroPoint,StartAngle - 5,HalfWidth);
          BackEndPt = RadialNSPointOpenClip(NSZeroPoint,EndAngle + 5,HalfWidth);
        }
        else {
          BackStartPt = RadialNSPointOpenClip(NSZeroPoint,StartAngle + 5,HalfWidth);
          BackEndPt = RadialNSPointOpenClip(NSZeroPoint,EndAngle - 5,HalfWidth);
        };
      };
        
      double RotationAngle = 0;
      if (GetRotationAngleOpenClip(TheObj,&RotationAngle))
        [t rotateByDegrees:RotationAngle];
        
      if (Width != Height)
        [t scaleXBy:1.0 yBy:(Height / Width)];
      
      NSBezierPath *ThePath = [NSBezierPath bezierPath];
      NSBezierPath *FillPath = [NSBezierPath bezierPath];
      
      if (IsArc) {
        [ThePath appendBezierPathWithArcWithCenter:NSZeroPoint
                                            radius:HalfWidth
                                        startAngle:StartAngle
                                          endAngle:EndAngle
                                         clockwise:IsClockwise];
        
        [FillPath moveToPoint:NSZeroPoint];
        [FillPath lineToPoint:StartPt];
        [FillPath appendBezierPathWithArcWithCenter:NSZeroPoint
                                            radius:HalfWidth
                                        startAngle:StartAngle
                                          endAngle:EndAngle
                                         clockwise:IsClockwise];
        [FillPath lineToPoint:NSZeroPoint];
      }
      else {
        [ThePath appendBezierPathWithOvalInRect:TheRect];
        [FillPath appendBezierPathWithOvalInRect:TheRect];
      };
      
      [ThePath transformUsingAffineTransform:t];
      
      if (GetFillColorAndSetFillOpenClip(TheObj)) {
        [FillPath transformUsingAffineTransform:t];
        [FillPath fill];
      };
      
      if (GetPenColorAndSetStrokeOpenClip(TheObj)) {
      };
      
      float PenSize = 1;
      GetPenSizeOpenClip(TheObj,&PenSize);
      GetLineDashAndSetPathDashOpenClip(TheObj,ThePath);
      [ThePath setLineWidth:PenSize];
      
      [ThePath stroke];

      if (IsArc) {
        StartPt = [t transformPoint:StartPt];
        EndPt = [t transformPoint:EndPt];
        BackStartPt = [t transformPoint:BackStartPt];
        BackEndPt = [t transformPoint:BackEndPt];
                
        double ArrowStartAngle = GetAngleOpenClip(BackStartPt,StartPt);
        double ArrowEndAngle = GetAngleOpenClip(BackEndPt,EndPt);
        
        [self drawArrows:TheObj StartPt:StartPt StartAngle:ArrowStartAngle EndPt:EndPt EndAngle:ArrowEndAngle];
      };
      
    };
    
  };  

};


- (void)drawPolygon:(NSDictionary *)TheObj
{
  NSArray *Points = [TheObj objectForKey:OpenClip_Points];
  
  if (Points) {
    NSBezierPath* ThePath = [NSBezierPath bezierPath];
    [ThePath setLineCapStyle:NSRoundLineCapStyle];
    
    NSPoint StartPt;
    NSPoint EndPt;
    double StartAngle;
    double EndAngle;
    
    int LastN = [Points count] - 1;

    for (int N = 0; (N <= LastN); N++) {
      NSPoint A;
      NSPoint LastPt;
      
      if (Get2DPointOpenClip(Points,N,&A)) {
        
        if (N == 0) {
          [ThePath moveToPoint:A];
          LastPt = A;
          StartPt = A;
        }
        else {
          [ThePath lineToPoint:A];
          
          if (N == 1)
            StartAngle = GetAngleOpenClip(A,LastPt);
          
          if (N == LastN) {
            EndAngle = GetAngleOpenClip(LastPt,A);
            EndPt = A;
          };
          
          LastPt = A;
        };
      
      };
    
    };
    
    BOOL Closed = NO;
    if (GetObjectClosedOpenClip(TheObj,&Closed)) {
      
      if (Closed)
        [ThePath closePath];
        
    };
    
    if (GetFillColorAndSetFillOpenClip(TheObj)) {
      [ThePath fill];
    };
      
    if (GetPenColorAndSetStrokeOpenClip(TheObj)) {
    };
    
    float PenSize = 1;
    GetPenSizeOpenClip(TheObj,&PenSize);
    GetLineDashAndSetPathDashOpenClip(TheObj,ThePath);
    [ThePath setLineWidth:PenSize];
    
    [ThePath stroke];
    
    if (! Closed)
      [self drawArrows:TheObj StartPt:StartPt StartAngle:StartAngle EndPt:EndPt EndAngle:EndAngle];
  };

};


- (void)drawBezier:(NSDictionary *)TheObj
{
  NSArray *Points = [TheObj objectForKey:OpenClip_Points];
  
  if (Points) {
    NSBezierPath* ThePath = [NSBezierPath bezierPath];
    [ThePath setLineCapStyle:NSRoundLineCapStyle];
    int Step = 1;
    NSPoint B;
    NSPoint C;
    NSPoint D;
    
    NSPoint StartPt;
    NSPoint EndPt;
    double StartAngle;
    double EndAngle;
    
    int LastN = [Points count] - 1;
    
    for (int N = 0; (N <= LastN); N++) {
      NSPoint ThePt;
      if (Get2DPointOpenClip(Points,N,&ThePt)) {
        
        if (N == 0) {
          [ThePath moveToPoint:ThePt];
          StartPt = ThePt;
        }
        else {
          
          if (N == 1)
            StartAngle = GetAngleOpenClip(ThePt,StartPt);
            
          switch (Step) {
            case 1 : B = ThePt;
                     Step = 2;
                     break;
            case 2 : C = ThePt;
                     Step = 3;
                     break;
            case 3 : D = ThePt;
                     [ThePath curveToPoint:D
                              controlPoint1:B
                              controlPoint2:C];
                     Step = 1;
                     
                     if (N == LastN) {
                       EndPt = D;
                       EndAngle = GetAngleOpenClip(C,D);
                     };
                     break;
          };
          
        };
      };
    };
    
    BOOL Closed = NO;
    if (GetObjectClosedOpenClip(TheObj,&Closed)) {
      
      if (Closed)
        [ThePath closePath];
        
    };
    
    if (GetFillColorAndSetFillOpenClip(TheObj)) {
      [ThePath fill];
    };
    
    if (GetPenColorAndSetStrokeOpenClip(TheObj)) {
    };
    
    float PenSize = 1;
    GetPenSizeOpenClip(TheObj,&PenSize);
    GetLineDashAndSetPathDashOpenClip(TheObj,ThePath);
    [ThePath setLineWidth:PenSize];
    
    [ThePath stroke];
    
    if (! Closed)
      [self drawArrows:TheObj StartPt:StartPt StartAngle:StartAngle EndPt:EndPt EndAngle:EndAngle];
  };
  
};


- (void)drawTheList:(NSArray *)TheNSList
{
  NSEnumerator *e = [TheNSList objectEnumerator];
  
  NSDictionary *TheObj;
  while (TheObj = [e nextObject]) {
    
    NSString *nType = [TheObj objectForKey:OpenClip_ObjectType];
    
    if (nType) {
      
      if ([nType isEqualToString:OpenClip_PointObj]) {
        NSArray *TheAdvancedList = [TheObj objectForKey:OpenClip_Advanced];
  
        if (TheAdvancedList) {
          [self drawTheAdvancedList:TheAdvancedList];  // draw both here
        };
        
        [self drawPoint:TheObj];
      }
      else if ([nType isEqualToString:OpenClip_LineObj]) {
        NSArray *TheAdvancedList = [TheObj objectForKey:OpenClip_Advanced];
  
        if (TheAdvancedList) {
          [self drawTheAdvancedList:TheAdvancedList];  // draw both here
        };
        
        [self drawLine:TheObj];
      }
      else if ([nType isEqualToString:OpenClip_RectObj]) {
        NSArray *TheAdvancedList = [TheObj objectForKey:OpenClip_Advanced];
  
        if (TheAdvancedList) {
          [self drawTheAdvancedList:TheAdvancedList];  // draw both here
        };
        
        [self drawRectangle:TheObj];
      }
      else if ([nType isEqualToString:OpenClip_EllipseObj]) {
        NSArray *TheAdvancedList = [TheObj objectForKey:OpenClip_Advanced];
  
        if (TheAdvancedList) {
          [self drawTheAdvancedList:TheAdvancedList];  // draw both here
        };
        
        [self drawEllipse:TheObj];
      }
      else if ([nType isEqualToString:OpenClip_PolygonObj]) {
        NSArray *TheAdvancedList = [TheObj objectForKey:OpenClip_Advanced];
  
        if (TheAdvancedList) {
          [self drawTheAdvancedList:TheAdvancedList];  // draw both here
        };
        
        [self drawPolygon:TheObj];
      }
      else if ([nType isEqualToString:OpenClip_BezierObj]) {
        
        NSArray *TheAdvancedList = [TheObj objectForKey:OpenClip_Advanced];
  
        if (TheAdvancedList) {
          [self drawTheAdvancedList:TheAdvancedList];  // draw both here
        };
        
        [self drawBezier:TheObj];
      }
      else if ([nType isEqualToString:OpenClip_GroupObj]) {
        
        NSArray *TheGroupList = [TheObj objectForKey:OpenClip_DrawingObjects];
  
        if (TheGroupList)
          [self drawTheList:TheGroupList];
      
      };
    
    };

  };

};


- (void)drawTheAdvancedList:(NSArray *)TheNSList
{
  NSEnumerator *e = [TheNSList objectEnumerator];
  
  NSDictionary *TheObj;
  while (TheObj = [e nextObject]) {
    
    NSString *nType = [TheObj objectForKey:OpenClip_ObjectType];
    
    if (nType) {
      
      if ([nType isEqualToString:OpenClip_BSplineObj])
        [self drawPolygon:TheObj];
    
    };

  };

};


- (void)drawTheLayers:(NSArray *)TheNSLayerList
{
  NSEnumerator *e = [TheNSLayerList objectEnumerator];
  
  NSDictionary *TheNSLayer;
  while (TheNSLayer = [e nextObject]) {
    
    NSString *TheLayerName;
    if (GetLayerNameOpenClip(TheNSLayer,TheLayerName)) {
    };
    
    float r,g,b,a;
    if (GetLayerColorOpenClip(TheNSLayer,&r,&g,&b,&a)) {
    };
    
    double TheLayerScale;
    if (GetLayerScaleOpenClip(TheNSLayer,&TheLayerScale)) {
    };
    
    // get layer drawing objects
    NSArray *TheNSList = [TheNSLayer objectForKey:OpenClip_DrawingObjects];
    
    if (TheNSList)
      [self drawTheList:TheNSList];
    
  };
  
};


- (void)writeToPasteboard
{
  PasteboardRef theClipboard;

  OSStatus err = PasteboardCreate(kPasteboardClipboard,&theClipboard);
  PasteboardClear(theClipboard);
  
  //PasteboardSyncFlags syncFlags;
  //syncFlags = PasteboardSynchronize( theClipboard ); // 2
  
  NSData *TheData = [NSKeyedArchiver archivedDataWithRootObject:mDocument];
  err = PasteboardPutItemFlavor(theClipboard,(PasteboardItemID)0,CFSTR("com.openclip.clip"),(CFDataRef)TheData,0);
};


- (BOOL)openFile:(NSString*)theFileName
{
  BOOL Result = NO;
  
  NSFileManager *fileManager = [NSFileManager defaultManager];
  NSArray *Paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory,NSLocalDomainMask,NO);
  
  if ([Paths count] > 0) {  
    NSString *destinationPath = [[Paths objectAtIndex:0] stringByAppendingPathComponent:@"OpenClip"];
    //NSLog(destinationPath); // /Library/Application Support/OpenClip
    
    if ([fileManager fileExistsAtPath:destinationPath]) {
      //NSLog(@"OpenClip Directory exists");
      
      NSString *TheFilePath = [destinationPath stringByAppendingPathComponent:theFileName];
      //NSLog(TheFilePath);  // /Library/Application Support/OpenClip/OpenClip
      
      NSFileHandle *theFileHandle = [NSFileHandle fileHandleForReadingAtPath:TheFilePath];
      
      NSDictionary *TheDocument = [NSDictionary dictionaryWithContentsOfFile:TheFilePath];
                      // [NSDictionary dictionaryWithContentsOfURL::ThePath];
      
      if (TheDocument) {
        mDocument = TheDocument;
        [mDocument retain];
        Result = YES;
      }
      
      [theFileHandle closeFile];
  
    };
    
  };
  
  return Result;

};


- (void)draw
{
  //NSLog(@"OpenClipDoc draw");
  
  double TheScale;
  if (GetScaleOpenClip(mDocument,&TheScale)) {
  };
  
  NSPoint GlobalOrigin;
  if (GetGlobalOriginOpenClip(mDocument,&GlobalOrigin)) {
  };
  
  NSPoint InsertionPoint;
  if (GetInsertionPointOpenClip(mDocument,&InsertionPoint)) {
  };
  
  NSArray *TheNSLayerList = [mDocument objectForKey:OpenClip_DrawingLayers];
  
  if (TheNSLayerList)
    [self drawTheLayers:TheNSLayerList];
  else {
    NSArray *TheNSList = [mDocument objectForKey:OpenClip_DrawingObjects];
    
    if (TheNSList)
      [self drawTheList:TheNSList];
  
  };

};


@end


//----------------------------------------------------------------------------------------------
