//----------------------------------------------------------------------------------------------

// OpenClipView.m

//----------------------------------------------------------------------------------------------

/*



*/

#import "OpenClipView.h"

#import "OpenClipDefines.h"
#import "NSBezierPathAdditions.h"
#import "OpenClipUtils.h"
#import "OpenClipDoc.h"

@implementation OpenClipView


- (id)initWithFrame:(NSRect)frame
{
  self = [super initWithFrame:frame];
  
  if (self) {
    //NSLog(@"OpenClipView initWithFrame");

    OpenClipDoc* ThisDoc = [[OpenClipDoc alloc] init];
    if ([ThisDoc openFile:@"OpenClip"]) {
      mDoc = [ThisDoc retain];
      //NSLog(@"OpenClipView opened file");
    };
  }
  return self;
}


- (void)dealloc
{
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  
  [mPrintInfo release];
  
  [super dealloc];
}


- (void)awakeFromNib
{
  [super awakeFromNib];
  [self setPrintInfo:[NSPrintInfo sharedPrintInfo]];
  
  NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
  
  [nc addObserver:self
         selector:@selector(updateChartSize:)
             name:@"DidChangePrintInfo"
           object:nil];
  
  //cursor = [NSCursor openHandCursor];
}


- (void)setDocument:(OpenClipDoc *)thisDoc
{
  if (mDoc != thisDoc) {
    [mDoc release];
    mDoc = [thisDoc retain];
  }
};


- (void)print:(id)sender
{
  //NSLog(@"OpenClipView print");
  [[NSPrintOperation printOperationWithView:self] runOperation];
}

// all this is from MPGChartView in Benchmark.  I think it's incomplete

- (void)updateFrame
{
  if ([self superview]) {
    //NSRect r = NSZeroRect;

    //r.size = [mPrintInfo paperSize];
    //[self setFrame:r];
  }
}


- (void)setPrintInfo:(NSPrintInfo *)p
{
  if (mPrintInfo != p) {
    [mPrintInfo autorelease];
    mPrintInfo = [p copyWithZone:[self zone]];
    [self updateFrame];
    [self setNeedsDisplay:YES];
  }
}


- (NSPrintInfo *)printInfo
{
  return mPrintInfo;
}


- (void)updateChartSize:(NSNotification *)n
{
  [self setNeedsDisplay:YES];
}


- (void)drawOpenClipLogo
{
  NSImage *Logo = [NSImage imageNamed:@"logo"];

  if (Logo) {
    NSRect TheRect = [self bounds];
    
    NSSize ImageSize = [Logo size];
    double HalfWidth  = (ImageSize.width / 2.0);
    double HalfHeight = (ImageSize.height / 2.0);
    double LLX = NSMidX(TheRect) - HalfWidth;
    double LLY = NSMidY(TheRect) - HalfHeight;
    
    NSRect sourceRect = {{0.0,0.0},[Logo size]};
    NSRect destRect = NSMakeRect(LLX,LLY,ImageSize.width,ImageSize.height);
  
    [Logo drawInRect:destRect
            fromRect:sourceRect
           operation:NSCompositeSourceOver  // or NSCompositeCopy
            fraction:1.0];
             
  }
  else
    NSLog(@"couldn't load image");

};


- (void)copyToPasteboard
{
  [mDoc writeToPasteboard];
};



- (void)drawRect:(NSRect)rect
{
  NSRect TheRect = [self bounds];
  
  //if ([[NSGraphicsContext currentContext] isDrawingToScreen]) {  // not printing
  if ([NSGraphicsContext currentContextDrawingToScreen]) {
    // draw screen-only elements here
    //NSEraseRect(TheRect);
    [[NSColor whiteColor] set];
    NSRectFill(TheRect);
  }
  else {
    // draw printer-only elements here
  };
  
  // draw common elements here
  
  if (mDoc) {
    [mDoc draw];
  }
  else {
    [self drawOpenClipLogo];
  };
  
  if ([NSGraphicsContext currentContextDrawingToScreen]) {
    [[NSColor grayColor] set];
    NSFrameRect(TheRect);
  };
}


- (BOOL)acceptsFirstResponder
{
  //NSLog(@"Accepting");
  return YES;
}


- (BOOL)resignFirstResponder
{
  //NSLog(@"Resigning");
  return YES;
}


- (BOOL)becomeFirstResponder
{
  //NSLog(@"Becoming");
  return YES;
}


@end
